SharedMemIf = [
        {
            'Name' : 'Term1',
            'Type' : 'f',
            'Offset' : 0,
            'Size' : 4
        },
        {
            'Name' : 'Term2',
            'Type' : 'f',
            'Offset' : 4,
            'Size' : 4
        },
        {
            'Name' : 'Term3',
            'Type' : 'f',
            'Offset' : 8,
            'Size' : 4
        },
        {
            'Name' : 'Term4',
            'Type' : 'f',
            'Offset' : 12,
            'Size' : 4
        },
        {
            'Name' : 'Term5',
            'Type' : 'f',
            'Offset' : 16,
            'Size' : 4
        }
    ]

memory_map = 'mem_files/mem_map_file'




import ctypes
import mmap
import os
from wicomm.SharedMemIf import SharedMemIf, memory_map
from wicomm.UdpDataSource import UdpDataSource

class SharedMemoryWriter:
    def __init__(self):
        self.data = {}
        self.dataSource = UdpDataSource()
        self.sharedMem = SharedMemIf
        self.mMapFile = memory_map
        self.initSharedMem()
        self.initData()

    def initSharedMem(self):
        fd = os.open(self.mMapFile, os.O_CREAT | os.O_TRUNC | os.O_RDWR)
        assert os.write(fd, '\x00' * mmap.PAGESIZE) == mmap.PAGESIZE
        self.buf = mmap.mmap(fd, mmap.PAGESIZE)

    def initData(self):
        for term in self.sharedMem:
            self.data[term["Name"]] = self.typeFromString(term["Type"]).from_buffer(self.buf, term["Offset"])

    def typeFromString(self, s):
        if s == 'f':
            return ctypes.c_float

    def update(self):
        unpacked_data = self.dataSource.update()
        #for term, value in zip(QuadSharedMem, unpacked_data):
        #    self.data[term["Name"]].value = value
        for term, value in zip(self.sharedMem, unpacked_data):
            self.data[term["Name"]].value = value



import mmap
import os
import struct
import time
from wicomm.SharedMemIf import SharedMemIf, memory_map

class SharedMemoryReader:
    def __init__(self):
        self.sharedMem = SharedMemIf
        self.mMapFile = memory_map
        self.initializeSharedMem()

    def initializeSharedMem(self):
        # Open the file for reading
        fd = os.open(self.mMapFile, os.O_RDWR)
        # Memory map the file
        self.buf = mmap.mmap(fd, mmap.PAGESIZE)

    def readData(self):
        returnDict = {}
        for term in self.sharedMem:
            returnDict[term["Name"]] = self.unloadData(self.buf, term)[0]
        return returnDict

    def unloadData(self, buf, term):
        return struct.unpack(term["Type"], buf[term["Offset"]:term["Offset"]+term["Size"]])



import socket
import json
import struct
from wicomm.WirelessDataSource import WirelessDataSource
from wicomm.SharedMemIf import SharedMemIf

# Define Quad UDP server host and port. This is the client.
HOST, PORT = "localhost", 2106

class UdpDataSource(WirelessDataSource):


    def __init__(self):
        # init base class
        super(UdpDataSource, self).__init__()
        # SOCK_DGRAM is the socket type to use for UDP sockets
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        self.host = HOST
        self.port = PORT
        self.apiId = {
            'ApiId' : 'UdpGateway'
        }


    def update(self):
        self.sock.sendto(json.dumps(self.apiId), (self.host, self.port))
        rcvdData = self.sock.recv(1024)
        # good debug print
        #print ":".join("{:02x}".format(ord(c)) for c in rcvdData)
        return struct.unpack_from(self.fmtString, rcvdData)



import socket
import json
import struct
from wicomm.SharedMemIf import SharedMemIf

# Define Quad UDP server host and port. This is the client.
HOST, PORT = "localhost", 2106

class WirelessDataSource(object):

    def __init__(self):
        self.fmtString = ''
        self.sharedMem = SharedMemIf
        self.convertToFmtString()

    def convertToFmtString(self):
        for sharedTerm in self.sharedMem:
            self.fmtString += sharedTerm["Type"]

    # Intended to be over-ridden in derived classes.
    def update(self):
        print "Wireless Data Source base class no implementation."



from setuptools import setup

def readme():
    with open('README.rst') as f:
        return f.read()

setup(name='wicomm',
      version='0.12',
      description='Python package containing a wireless communication API to be used in robotics applications.',
      long_description='Provides a framework to receive data over a socket and pump it into a shared memory.',
      classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
        'Topic :: Text Processing :: Linguistic',
      ],
      keywords='wicomm',
      url='http://hardworkin-man.com:8080/scm/git/wicomm',
      author='Mike Moore',
      author_email='mickety.mike@gmail.com',
      license='MIT',
      packages=['wicomm'],
      install_requires=[
          #'markdown',
      ],
      test_suite='nose.collector',
      tests_require=['nose'],
      scripts=['bin/ReadFromSharedMem','bin/WriteToSharedMem'],
      include_package_data=True,
      zip_safe=False)